/*
 * linux/arch/arm/drivers/sound/soundcont.c
 *
 * Copyright (C) 1994, 1995, 1996 Russell King
 *
 * Control various sound parameters.
 *
 */

#include <linux/errno.h>
#include "soundinit.h"
#include "soundcont.h"

#include "asm/system.h"

#define MAX(x,y)	(x) < (y) ? (y) : (x)

int sound_sampleperiod;			/* our *real* idea of no. of us between sample o/ps */
unsigned char sound_allocated[N_CHANNELS];
#define VIDC_SAMPLERATE(x) (((x)*3)>>1)

/*
 * Set the sample period
 */

int sound_configure(int period, int nchannels, int *oldperiod, int *oldnchannels)
{
	int snew=0, vnew=0, cnew=sound_lognchannels;
	unsigned long flags;
	
	if(oldperiod)
		*oldperiod = sound_sampleperiod << sound_lognchannels;
	if(oldnchannels)
		*oldnchannels = 1<<sound_lognchannels;
	
	if(nchannels != 0)
	{
		switch(nchannels)
		{
			case 1: cnew = 0; break;
			case 2: cnew = 1; break;
			case 3:
			case 4: cnew = 2; break;
			case 5:
			case 6:
			case 7:
			case 8: cnew = 3; break;
			default: return -EINVAL;
		}
		if(period == 0)
			period = sound_sampleperiod << sound_lognchannels;
	}
	
	save_flags(flags);
	
	if(period != 0)
	{
		snew=period>>cnew;
		vnew=VIDC_SAMPLERATE(snew);
		if(vnew < 3 || vnew > 255)
			return -EINVAL;

		cli();
		sound_sampleperiod = snew;
		sound_vidcsampleperiod = vnew|0x100;
		sound_toupdate|=SOUND_SNDFREQ;
	}
	
	if(cnew != sound_lognchannels)
		sound_lognchannels = cnew;
	restore_flags(flags);
	
	return 0;
}

/*
 * Set the overall volume of the system.
 * This only works if the voice generator is using the tables
 */
int sound_vol(int volume)
{
	int i,thislen,len,sample,p,n;
	int old;
	unsigned char *bp,*ep;

	if(volume==-1)
		return sound_volume;

	if(volume<0 || volume>127)
		return -EINVAL;

	old=sound_volume;

	sound_volume=volume;

	volume = (127-(volume & 127))<<1;

	for(i = 0; i < 256; i++)
		table_log[i] = MAX(i-volume, 0);

	bp = table_linsigned;
	ep = bp+8192;

	*bp++ = 0;

	len = 1;
	sample = 0;

	while(1)
	{
		n=15;
		do
		{
			thislen = len;
			sample+=2;
			if(sample>254)
				sample=254;

			p=MAX(sample-volume,0);
			/* Insert samples to cover this minor block */
			do
			{
				*bp++ = p;
				*--ep = p | 1;
				if(bp >= ep)
					goto end;
			}
			while(--thislen > 0);
		}
		while(--n>=0);
		len <<= 1;
	}
end:
	return old;
}

int sound_setimage(int channel, int image)
{
	int i,n=1 << sound_lognchannels;

	for(i=channel; i<N_CHANNELS; i+=n) {
		sound_stereoimage[i] = image;
	}
	sound_toupdate |= SOUND_IMAGE;
	return 0;
}
