/*
 * soundinit.h
 *
 * These are the structures used in initialising and maintaining the
 * sound system.
 */

#ifndef __sound_soundinit_h
#define __sound_soundinit_h

#ifndef __sound_config_h
#include "config.h"
#endif

/*
 * This structure contains the info for a /dev device.
 */
struct sound_ops
{
  const char *name;
  struct file_operations *fops;
  void *data;
};

/*
 * This is the log of the current number of channels currently configured.
 */
extern int sound_lognchannels;

/*
 * This is the sample length for one sample in the output buffer
 */
extern int sound_samplelength;

/*
 * This is the output period. (Multiply by no. of channels for sample period)
 */
extern int sound_sampleperiod;

extern int sound_vidcsampleperiod;

/*
 * This is the current overall volume of the sound system.
 */
extern int sound_volume;

/*
 * This is to tell the interrupt routine to reprogram certain registers
 */
extern int sound_toupdate;
#define SOUND_SNDFREQ		(1<<0)
#define SOUND_IMAGE		(1<<1)

/*
 * This contains the current stereo positions of each 'output' channel.
 */
extern int sound_stereoimage[];
#define SOUND_LEFT		32
#define SOUND_83LEFT		64
#define SOUND_67LEFT		96
#define SOUND_CENTRE		128
#define SOUND_67RIGHT		160
#define SOUND_83RIGHT		192
#define SOUND_RIGHT		224

extern int register_sounddev(unsigned int minor, const char *name,
	struct file_operations *fops, void *data);
extern int unregister_sounddev(unsigned int minor, const char *name);

#endif
