/*
 * soundirq.h
 *
 * Handle sound system interrupts
 *
 */

#ifndef __sound_soundirq_h
#define __sound_soundirq_h

#ifndef __sound_config_h
#include "config.h"
#endif

/*
 * Process the current sound interrupt, and swap internal buffers
 */
extern void sound_irq(int irq_nr, void *dev_id, struct pt_regs *regs);

/*
 * Data structure used by interrupt routine to call the low-level
 * code in the voice generators to re-fill the output buffer.
 */
struct voice
{
  /* The fill routine */
  void (*fill)(char *bufstart,char *bufend,char increment);
  void (*gateon)(struct voice *);
  void (*gateoff)(struct voice *);
  /* Other data */
  int  data[8];
};

/*
 * This is a quiet voice generator
 *
 * This is used when a voice generator has been removed from a
 * channel.
 */
extern struct voice sound_quiet_voice;
extern struct voice sound_to_quiet_voice;

/*
 * This is the data used by the interrupt routine to call the voice generators
 */
extern struct voice *sound_voice[];

#endif
