/*
 * voice.h
 *
 * Voice generator header file
 *
 */

#ifndef __sound_voice_h
#define __sound_voice_h

#ifndef __sound_config_h
#include "config.h"
#endif

#ifndef __sound_soundirq_h
#include "soundirq.h"
#endif

/*
 * The structure voice_driver is hard coded into voice.s.
 * Don't add anything before the voice data comment without
 * changing voice.s.
 */

struct voice_driver
{
/* Used by irq - level routines */
	char *vd_buf;				/* Computed sample waveform buffer / index */
	int vd_cnt;				/* Number of samples left in buffer */
#define vd_dur vd_cnt
	int vd_tail;				/* Tail of circulat buffer */
#define vd_phase vd_tail
	int vd_volume;				/* This voice's volume */
	const unsigned char *vd_convtable;	/* Conversion table (can be wave table) */
	int vd_size;				/* Size of waveform buffer (power of 2) */
	int vd_bits;				/* Bits per sample */
	struct wait_queue *vd_wait;		/* Wait queue for writes */
/* Voice data */
	int vd_head;				/* Head of circular buffer */
	int vd_channel;				/* Channel that this voice is attached */
	int (*vd_write)(struct voice_driver *vd, const char *b, int nr);
						/* Writing routine */
	int (*vd_alloc)(int data);		/* Allocation routine */
	int (*vd_install)(struct voice_driver *vd, struct voice *vdrv);
						/* Install voice driver */
	void (*vd_remove)(struct voice_driver *vd, struct voice *vdrv);
						/* Remove a voice from a channel */
	int (*vd_vol)(struct voice_driver *vd,int newvol);
						/* Change channel volume */
};

/*
 * Allocate a voice generator
 * type: see below
 * data: Unused at the moment,
 */
extern int voice_allocate(int type,int data);
#define VOICE_SAMPLE	1
#define VOICE_WAVE	2
#define VOICE_SINE	3

/*
 * Reallocate the buffer used by the voice generator
 */
extern int voice_realloc(struct voice_driver *vd, int newsize);

/*
 * Install a voice generator on a channel
 * This doesn't enable the channel though!
 */
extern int voice_install(struct voice_driver *vd,int channel);

/*
 * find a free channel or -1 on error (channel = -1 to find, channel no to check)
 */
extern int voice_findfree(int channel);

/*
 * Remove a voice generator from a channel,
 * installing a quiet voice generator.
 */
extern void voice_remove(struct voice_driver *vd);

/*
 * Release a voice generator (installs a quiet generator)
 * and free its memory
 */
extern int voice_release(struct voice_driver *vd);

/*
 * Change the volume on a channel
 */
extern int voice_volume(struct voice_driver *vd, int newvolume);

#endif
