/*
 * linux/arch/arm/drivers/sound/voice_wave.c
 *
 * Copyright (C) 1994, 1995, 1996 Russell King
 *
 * Wave table voice driver.
 */

#include <linux/sched.h>
#include <linux/errno.h>
#include <linux/malloc.h>
#include "soundinit.h"
#include "soundirq.h"
#include "voice.h"

#include <asm/segment.h>

#ifndef MIN
#define MIN(a,b)	((a) < (b) ? (a) : (b))
#endif

static int voice_wave_write(struct voice_driver *vd, const char *b, int count)
{
	int c, total =0;
	unsigned long flags;

	save_flags(flags);
	while(1) {
		cli();
		c = MIN(count, MIN(vd->vd_size - vd->vd_cnt - 1,
				   vd->vd_size - vd->vd_head));
		if(c<=0)
			break;

		memcpy_fromfs(vd->vd_buf + vd->vd_head, b, c);
		vd->vd_head = (vd->vd_head + c) & (vd->vd_size-1);
		vd->vd_cnt += c;
		restore_flags(flags);
		b += c;
		count -= c;
		total += c;
	}
	restore_flags(flags);
	return total;
}

static int voice_wave_alloc(int data)
{
	struct voice_driver *vd;
	extern struct voice_driver voice_wave;

	vd = (struct voice_driver *)kmalloc(sizeof(struct voice_driver), GFP_KERNEL);
	if(!vd)
		return -ENOMEM;

	memcpy(vd, &voice_wave, sizeof(struct voice_driver));

	vd->vd_buf = NULL;
	return (int)vd;
}

static void voice_wave_gateoff(struct voice *vdrv)
{
	struct voice_driver *vd;
	
	vd = (struct voice_driver *)vdrv->data[0];
	if(!vd->vd_buf)
		voice_release(vd);
}

static int voice_wave_install(struct voice_driver *vd, struct voice *vdrv)
{
	extern void voice_driver_wave(char *, char *, char);
	
	vdrv->fill	= voice_driver_wave;
	vdrv->gateon	= NULL;
	vdrv->gateoff	= voice_wave_gateoff;
	vdrv->data[0]	= (int)vd;
	
	return 0;
}

unsigned char voice_wave_sine[]=
{
	0x00, 0x5A, 0x76, 0x88, 0x96, 0xA2, 0xA8, 0xAE,
	0xB4, 0xBA, 0xC0, 0xC4, 0xC6, 0xCA, 0xCC, 0xD0,
	0xD2, 0xD6, 0xD8, 0xDC, 0xDE, 0xE0, 0xE2, 0xE4,
	0xE4, 0xE6, 0xE8, 0xE8, 0xEA, 0xEC, 0xEC, 0xEE,
	0xEE, 0xF0, 0xF0, 0xF2, 0xF2, 0xF4, 0xF4, 0xF6,
	0xF6, 0xF8, 0xF8, 0xF8, 0xFA, 0xFA, 0xFC, 0xFC,
	0xFC, 0xFC, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE,
	0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE,
	0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE,
	0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFE, 0xFC,
	0xFC, 0xFC, 0xFC, 0xFA, 0xFA, 0xF8, 0xF8, 0xF8,
	0xF6, 0xF6, 0xF4, 0xF4, 0xF2, 0xF2, 0xF0, 0xF0,
	0xEE, 0xEE, 0xEC, 0xEC, 0xEA, 0xE8, 0xE8, 0xE6,
	0xE4, 0xE4, 0xE2, 0xE0, 0xDE, 0xDC, 0xD8, 0xD6,
	0xD2, 0xD0, 0xCC, 0xCA, 0xC6, 0xC4, 0xC0, 0xBA,
	0xB4, 0xAE, 0xA8, 0xA2, 0x96, 0x88, 0x76, 0x5A,
	0x00, 0x5D, 0x79, 0x89, 0x97, 0xA3, 0xA9, 0xAF,
	0xB5, 0xBB, 0xC1, 0xC5, 0xC7, 0xCB, 0xCD, 0xD1,
	0xD3, 0xD7, 0xD9, 0xDD, 0xDF, 0xE1, 0xE3, 0xE5,
	0xE5, 0xE7, 0xE9, 0xE9, 0xEB, 0xED, 0xED, 0xEF,
	0xEF, 0xF1, 0xF1, 0xF3, 0xF3, 0xF5, 0xF5, 0xF7,
	0xF7, 0xF9, 0xF9, 0xF9, 0xFB, 0xFB, 0xFD, 0xFD,
	0xFD, 0xFD, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFD,
	0xFD, 0xFD, 0xFD, 0xFB, 0xFB, 0xF9, 0xF9, 0xF9,
	0xF7, 0xF7, 0xF5, 0xF5, 0xF3, 0xF3, 0xF1, 0xF1,
	0xEF, 0xEF, 0xED, 0xED, 0xEB, 0xE9, 0xE9, 0xE7,
	0xE5, 0xE5, 0xE3, 0xE1, 0xDF, 0xDD, 0xD9, 0xD7,
	0xD3, 0xD1, 0xCD, 0xCB, 0xC7, 0xC5, 0xC1, 0xBB,
	0xB5, 0xAF, 0xA9, 0xA3, 0x97, 0x89, 0x79, 0x5D
};

const struct voice_driver voice_wave =
{
	NULL,			/* Buffer (release at end/don't release) */
	0,			/* Count  (duration) */
	0,			/* Tail   (phase accumulator) */
	127,			/* Volume */
	voice_wave_sine,	/* Wave table */
	4096,			/* Buffer size */
	8,			/* Bits/sample */
	NULL,			/* Wait queue */
	0,			/* Head */
	-1,			/* Channel */
	voice_wave_write,	/* Write sample data */
	voice_wave_alloc,	/* Allocate new voice */
	voice_wave_install,	/* Install voice driver */
	NULL,			/* Remove voice driver (default) */
	NULL			/* Change channel volume */
};
