/*
 * linux/arch/arm/kernel/head32.S
 *
 * Copyright (C) 1994, 1995, 1996, 1997 Russell King
 *
 * Kernel 32 bit startup code for ARM6 / ARM7 / StrongARM
 */
#include <linux/config.h>

		.text
		.align

		.globl	_swapper_pg_dir
		.equ	_swapper_pg_dir,	0xc0008000

		.globl	__entry
		.globl	__stext
/*
 * Entry point and restart point.  Entry *must* be called with r0 == 0,
 * MMU off.
 *
 *  r1 = 0 -> ebsa  (Ram @ 0x00000000)
 *  r1 = 1 -> RPC   (Ram @ 0x10000000)
 *  r1 = 2 -> ebsit (???)
 */

#define DC21285_ARMCSR_BASE             0x42000000
#define	CSR_BASE			DC21285_ARMCSR_BASE

#define IRQ_ENABLE_CLEAR                0x18C 
#define FIQ_ENABLE_CLEAR                0x28C 

#define DC21285_XBUS_XCS2               0x40012000

#define XBUS_DEV_2                      DC21285_XBUS_XCS2

#define LED_BANK                        XBUS_DEV_2
#define RED_LED                         0x04
#define GREEN_LED                       0x02
#define YELLOW_LED                      0x01

#define	IO_BASE_PHYSICAL		0x40000000
#define IO_BASE_VIRTUAL			0xE0000000

__stext:
__entry:
@
@ Tell the world we've arrived
@
		LDR	r5, =(RED_LED + GREEN_LED + YELLOW_LED)
		EOR	r5, r5, #7
		LDR	r6, =LED_BANK
		STR	r5, [r6]

		BL	resetCOM
	
		mov	r0, #'\n'
		bl	_printchP
		mov	r0, #'\r'
		bl	_printchP
		mov	r0, #'a'
		bl	_printchP
		mov	r0, #'.'
		bl	_printchP

@ ----------------------------------------------------------------------
@ Deal with being loaded by Angel/Demon
@
@
@
@ Disable clock switching
@
		mcr	p15, 0, r0, c15, c2 ,2

@
@ Turn off interrupts from the 21285
@
		ldr	r0, =0xFFFFFFFF
		ldr	r1, =CSR_BASE
		str	r0, [r1, #IRQ_ENABLE_CLEAR]
		str	r0, [r1, #FIQ_ENABLE_CLEAR]

@
@ disable the i-cache
@
		mov	r0, #0			@ Disable MMU & caches
		mcr	p15, 0, r0, c1, c0	
		mcr	p15, 0, r0, c8, c7	@ flush I,D TLBs
		mcr	p15, 0, r0, c7, c7	@ flush I,D caches
		nop				@ make sure that there are
		nop				@ no pipelined instructions
		nop
		nop
	
		mov	r0, #'b'
		bl	_printchP
		mov	r0, #'.'
		bl	_printchP
@
@ Map physical memory to virtual for the first 8Mbytes
@
		mov	r0, r4
		orr	r1, r5, #0x00000002	@ Physical memory | TYPE_SECT
		orr	r1, r1, #0x00000c00	@ AP_READ | AP_WRITE
		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		str	r1, [r0], #4

		bl	_printchP
		mov	r0, #'c'
		bl	_printchP
		mov	r0, #'.'
		bl	_printchP

@
@ end of angel/demon stuff
@ ----------------------------------------------------------------------
@
@ Fudge the types (for now)
@	
		mov	r0, #0
		mov	r1, #0
@
@ First thing to do is to get the page tables set up so that we can call the kernel
@ in the correct place.  This is relocatable code...
@
		adr	r4, LCtypes
		add	r4, r4, r1, lsl #4
		ldmia	r4, {r4, r5, r6}	@ r4 = page dir in physical ram
		mov	r0, r4
		mov	r1, #0
		add	r2, r0, #0x4000
1:		str	r1, [r0], #4		@ Clear page table
		teq	r0, r2
		bne	1b

@
@ Add enough entries to allow the kernel to be called.
@ It will sort out the real mapping in paging_init
@
@ Map the physical memory to va 0xC0000000
@
		add	r0, r4, #0x3000
		mov	r1, #0x0000000e		@ SECT_CACHEABLE | SECT_BUFFERABLE | TYPE_SECT
		orr	r1, r1, #0x00000c00	@ AP_READ | AP_WRITE
		add	r1, r1, r5
		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		str	r1, [r0], #4
		add	r1, r1, #1 << 20
@
@ Map in IO space (to va 0xE0000000)
@
		add	r0, r4, #0x3800
		orr	r1, r6, #0x00000002	@ IO_Space | TYPE_SECT
		orr	r1, r1, #0x00000c00	@ AP_READ | AP_WRITE
		add	r2, r0, #0x0800
1:		str	r1, [r0], #4
		add	r1, r1, #1 << 20
		teq	r0, r2
		bne	1b
@
@ Map in screen at 0x02000000 & SCREEN2_BASE
@
		teq	r5, #0
		addne	r0, r4, #0x80			@ 02000000
		movne	r1, #0x02000000
		orrne	r1, r1, #0x00000c00
		orrne	r1, r1, #0x00000002
		strne	r1, [r0]
		addne	r0, r4, #0x3600			@ d8000000
		strne	r1, [r0]

@
@ Go to red, don't touch r4, that contains the page table address in physical memory
@
		LDR	r5, =RED_LED
		EOR	r5, r5, #7
		LDR	r6, =LED_BANK
		STR	r5, [r6]

@
@ The following should work on both v3 and v4 implementations
@
		mov	r0, #0
		mrc	p15, 0, r1, c0, c0	@ get ID
		eor	r2, r1, #0x41000000
		eor	r2, r2, #0x00560000
		eors	r2, r2, #0x00000600	@ ARM6 special case
		tstne	r1, #0x00ff0000		@ r1b24-16 = architecture version
		mcreq	p15, 0, r0, c7, c0	@ flush caches on v3
		mcreq	p15, 0, r0, c5, c0	@ flush TLBs on v3
		mcrne	p15, 0, r0, c7, c7	@ flush I,D caches on v4
		mcrne	p15, 0, r0, c7, c10, 4	@ drain write buffer on v4
		mcrne	p15, 0, r0, c8, c7	@ flush I,D TLBs on v4
		mcr	p15, 0, r4, c2, c0	@ load page table pointer
		mov	r0, #5			@ Domains 0, 1 = client
		mcr	p15, 0, r0, c3, c0	@ load domain access register
		mrcne	p15, 0, r0, c1, c0	@ get control register v4
		orr	r0, r0, #0x35		@ ....DP.C.M (no write buffer)
@		orr	r0, r0, #0x3d		@ ....DPWC.M
		orrne	r0, r0, #1 << 12	@ v4 supports separate I cache
		ldr	lr, Lbranch
		mcr	p15, 0, r0, c1, c0	@ Enable MMU & caches.  In 3 instructions
						@ we loose this page!
		mov	pc, lr
Lbranch:	.long	Lalready_done_mmap

		@ EBSA285 (pg dir phys, phys ram start, phys i/o)
LCtypes:	.long	_swapper_pg_dir - 0xc0000000
		.long	0
		.long	IO_BASE_PHYSICAL
		.long	0

		@ RPC
		.long	_swapper_pg_dir - 0xc0000000 + 0x10000000
		.long	0x10000000
		.long	0x03000000
		.long	0

		@ EBSIT ???
		.long	_swapper_pg_dir - 0xc0000000
		.long	0
		.long	0xe0000000
		.long	0

LC0:		.long	_edata
		.long	_arm_id
		.long	_end
		.long	_init_user_stack + 4096
		.align

Lalready_done_mmap:
@
@ Go to YELLOW (use virtual addresses)
@
		LDR	r5, =(YELLOW_LED)
		EOR	r5, r5, #7
		LDR	r6, =(LED_BANK - IO_BASE_PHYSICAL + IO_BASE_VIRTUAL)
		STR	r5, [r6]

		adr	r5, LC0
		ldmia	r5, {r5, r6, sl, sp}	@ Setup stack
		ldr	r6, [r6]
		mov	r4, #0
1:		cmp	r5, sl			@ Clear BSS
		strcc	r4, [r5],#4
		bcc	1b
@
@ Read processor ID register (CP#15, CR0).
@ NOTE: ARM2 & ARM250 cause an undefined instruction exception...
@ Values are:
@		XX01XXXX = ARMv4 architecture (StrongARM)
@		XX00XXXX = ARMv3 architecture
@		4156061X = ARM 610
@ 		4156030X = ARM 3
@		4156025X = ARM 250
@		4156020X = ARM 2
@
		mrc	p15, 0, r5, c0, c0	@ Get processor ID
		eor	r0, r5, #0x44000000
		eor	r0, r0, #0x00010000
		eor	r0, r0, #0x0000a100
		bics	r0, r0, #15		@ Check for SA-110
		mcreq	p15, 0, r4, c15, c1, 2	@ Clock switching enabled!
		str	r5, [r6]
		mov	fp, #0
	
@
@ Go to green
@
		LDR	r5, =(GREEN_LED)
		EOR	r5, r5, #7
		LDR	r6, =(LED_BANK - IO_BASE_PHYSICAL + IO_BASE_VIRTUAL)
		STR	r5, [r6]

		mov	r0, #'d'
		bl	_putchar
		mov	r0, #'.'
		bl	_putchar

		b	_start_kernel

@ debug stuff HACK
@ UART registers

#define COM1Port                0x3f8
#define COM2Port                0x2f8		
#define IOShift                 2

#define DC21285_XBUS_XCS1       0x40011000

#define XBUS_DEV_1              DC21285_XBUS_XCS1

#define	COM1			(XBUS_DEV_1 + (COM1Port << IOShift))
#define COM2			(XBUS_DEV_1 + (COM2Port << IOShift))

#define Rx			0
#define Tx			0
#define IntEnable		0x4
#define FIFOcntl		0x8
#define LineCntl		0xC
#define ModemCntl		0x10
#define LineStatus              0x14

#define DLABMsk			0x80
#define LineTEMTMsk		0x40
	
#define Baud9600low		12
#define Baud9600high		0

#define WordLen8		3

#define Dllsb			0
#define Dlmsb			4            


		.globl	_resetCOM
resetCOM:	LDR	r2,=COM2
		ldr	r1, =0			@ disable interrupt 
		strb	r1, [r2, #IntEnable]
		ldr	r1, =DLABMsk		@ Set divisor access
		strb	r1, [r2, #LineCntl]
		ldr	r1, =Baud9600low	@ Set 9600 low divisor
		strb	r1, [r2, #Dllsb]
		ldr	r1, =Baud9600high	@ set 9600 high divisor
		strb	r1, [r2, #Dlmsb]
		ldr	r1, =WordLen8		@ Clear divisor access
		strb	r1, [r2, #LineCntl]
		ldr	r1, =0			@ 0 disables the FIFOs
		strb	r1, [r2, #FIFOcntl]
		ldrb	r1, [r2, #LineStatus]	@ read status will clear errors
		ldr	r1, =0xf
		strb	r1, [r2, #ModemCntl]	@ disable modem control
                MOV     pc, lr

		.globl	_printchV	@ virtual
_printchV:	ldr	r3, =(COM2 - IO_BASE_PHYSICAL + IO_BASE_VIRTUAL)
		b	_printcharacter
		.globl	_printchP	@ physical
_printchP:	ldr	r3, =COM2
_printcharacter:
4:		ldrb    r2,[r3,#LineStatus]
		tst     r2,#LineTEMTMsk         @ Wait until ready 
		beq     4b
		strb    r0,[r3,#Tx]
		mov	pc, lr

@
@ Virtual led writing routine
@
		.globl _putLEDs
_putLEDs:
		stmfd	sp!, {r1}
		eor	r0, r0, #7
		ldr	r1, =(LED_BANK - IO_BASE_PHYSICAL + IO_BASE_VIRTUAL)
		str	r0, [r1]
		ldmfd	sp!, {r1}
		mov	pc, lr
		
@
@ Virtual character writing routine
@
		.globl _putchar
_putchar:
		stmfd	sp!, {r2, r3, lr}
		bl	_printchV
		ldmfd	sp!, {r2, r3, pc}
		
		.bss
hexbuf:		.space 16


