/*
 * linux/arch/arm/mm/arm6.S: MMU functions for ARM6
 *
 * (C) 1997 Russell King
 *
 * These are the low level assembler for performing cache and TLB
 * functions on the ARM6 & ARM7.
 */

#include <asm/assembler.h>
#include "constants.h"

/*
 * Function: arm6_flush_cache_all (void)
 *	   : arm6_flush_cache_page (unsigned long address, int size, int flags)
 *
 * Params  : address	Area start address
 *	   : size	size of area
 *	   : flags	b0 = I cache as well
 *
 * Purpose : Flush all cache lines
 */
_arm6_flush_cache:
		mov	r0, #0
		mcr	p15, 0, r0, c7, c0, 0		@ flush cache
_arm6_null:
		mov	pc, lr

/*
 * Function: arm6_flush_tlb_all (void)
 *
 * Purpose : flush all TLB entries in all caches
 */
_arm6_flush_tlb_all:
		mov	r0, #0
		mcr	p15, 0, r0, c5, c0, 0		@ flush TLB
		mov	pc, lr

/*
 * Function: arm6_flush_tlb_page (unsigned long address, int size, int flags)
 *
 * Params  : address	Area start address
 *	   : size	size of area
 *	   : flags	b0 = I cache as well
 *
 * Purpose : flush a TLB entry
 */
_arm6_flush_tlb_area:
1:		subs	r1, r1, #4096
		mcr	p15, 0, r0, c6, c0, 0		@ flush TLB
		addne	r0, r0, #4096
		bne	1b
		mov	pc, lr

LC0:		.word	_current
/*
 * Function: arm6_switch_to (struct task_struct *prev, struct task_struct *next)
 *
 * Params  : prev	Old task structure
 *	   : next	New task structure for process to run
 *
 * Purpose : Perform a task switch, saving the old processes state, and restoring
 *	     the new.
 *
 * Notes   : We don't fiddle with the FP registers here - we postpone this until
 *	     the new task actually uses FP.  This way, we don't swap FP for tasks
 *	     that do not require it.
 */
_arm6_switch_to:
		stmfd	sp!, {r4 - r9, fp, lr}		@ Store most regs on stack
		mrs	ip, cpsr
		stmfd	sp!, {ip}			@ Save cpsr_SVC
		add	r0, r0, #TSS_SAVE		@ Save point
		str	sp, [r0]			@ Save sp_SVC
		ldr	r2, LC0
		str	r1, [r2]
		add	r2, r1, #TSS_SAVE
		ldr	sp, [r2]			@ Get saved sp_SVC
		ldr	r0, [r1, #TSS_MEMMAP]		@ Page table pointer
		mov	r1, #0
		mcr	p15, 0, r1, c7, c0, 0		@ flush cache
		mcr	p15, 0, r0, c2, c0, 0		@ update page table ptr
		mcr	p15, 0, r1, c5, c0, 0		@ flush TLBs
		ldmfd	sp!, {ip}
		msr	spsr, ip			@ Save tasks CPSR into SPSR for this return
		ldmfd	sp!, {r4 - r9, fp, pc}^		@ Load all regs saved previously

/*
 * Function: arm6_data_abort ()
 *
 * Params  : r0 = address of aborted instruction
 *
 * Purpose : obtain information about current aborted instruction
 *
 * Returns : r0 = address of abort
 *	   : r1 = FSR
 *	   : r2 != 0 if writing
 * Notes   : data abort *** NEEDS WORK ***
 */
_arm6_data_abort:
		ldr	r2, [r0]			@ read instruction causing problem
		and	r2, r2, #1 << 20		@ check write bit
		mrc	p15, 0, r0, c6, c0, 0		@ get FAR
		mrc	p15, 0, r1, c5, c0, 0		@ get FSR
		and	r1, r1, #15
		mov	pc, lr

/*
 * Function: arm6_check_bugs (void)
 *	   : arm6_proc_init (void)
 *	   : arm6_proc_fin (void)
 *
 * Notes   : This processor does not require these
 */
_arm6_check_bugs:
		mrs	ip, cpsr
		bic	ip, ip, #F_BIT
		msr	cpsr, ip
_arm6_proc_init:
_arm6_proc_fin:
		mov	pc, lr

/*
 * Purpose : Function pointers used to access above functions - all calls
 *	     come through these
 */
_arm6_name:	.ascii	"arm6\0"
		.align

		.globl	_arm6_processor_functions
_arm6_processor_functions:
		.word	_arm6_name			@  0
		.word	_arm6_switch_to			@  4
		.word	_arm6_data_abort		@  8
		.word	_arm6_check_bugs		@ 12
		.word	_arm6_proc_init			@ 16
		.word	_arm6_proc_fin			@ 20

		.word	_arm6_flush_cache		@ 24
		.word	_arm6_flush_cache		@ 28
		.word	_arm6_null			@ 32
		.word	_arm6_null			@ 36
		.word	_arm6_flush_cache		@ 40
		.word	_arm6_flush_tlb_all		@ 44
		.word	_arm6_flush_tlb_area		@ 48
