/*
 * linux/include/asm-arm/io.h
 *
 * Copyright (C) 1996 Russell King
 *
 * Modifications:
 *  16-Sep-1996	RMK	Inlined the inx/outx functions & optimised for both
 *			constant addresses and variable addresses.
 */
#ifndef __ASM_ARM_IO_H
#define __ASM_ARM_IO_H

#include <asm/hardware.h>
#if 0
#include <asm/proc/io.h>
#endif

/*
 * IO -> physical
 */
#define virt_to_bus(x) x
#define bus_to_virt(x) x

#define __ADDR_PCIO_MAX		65536
#define __PORT_PCIO(x)		(!((x) & 0x80000000))

/*
 * These select whether we use the inline versions.  We're not quite ready for
 * the inline ASM versions, but the constant port versions are fine.
 */
#undef  USE_INLINE_ASM
#define USE_INLINE_ASM1
#define USE_INLINE_CONST

#ifdef USE_INLINE_CONST
/*
 * These have to be macros for the 'J' constraint to work - +/-4096 immediate
 * operand.
 */
#define __outbc(value,port)						\
({									\
	if (__PORT_PCIO(port))						\
		__asm__ __volatile__(					\
		"strb	%0, [%1, %2]"					\
		: : "r" (value), "r" (PCIO_BASE), "Jr" (port << 2));	\
	else								\
		__asm__ __volatile__(					\
		"strb	%0, [%1, %2]"					\
		: : "r" (value), "r" (IO_BASE), "r" (port << 2));	\
})

#ifdef __HAVE_ASM_HALFWORD_LDST
#define __outwc(value,port)						\
({									\
	if (port < 256)							\
		__asm__ __volatile__(					\
		"strh	%0, [%1, %2]"					\
		: : "r" (value), "r" (PCIO_BASE), "J" (port << 2));	\
	else if (__PORT_PCIO(port))					\
		__asm__ __volatile__(					\
		"strh	%0, [%1, %2]"					\
		: : "r" (value), "r" (PCIO_BASE), "r" (port << 2));	\
	else								\
		__asm__ __volatile__(					\
		"strh	%0, [%1, %2]"					\
		: : "r" (value), "r" (IO_BASE), "r" (port << 2));	\
})
#else
#define __outwc(value,port)						\
({									\
	unsigned long v = value;					\
	if (__PORT_PCIO(port))						\
		__asm__ __volatile__(					\
		"str	%0, [%1, %2]"					\
		: : "r" (v|v<<16), "r" (PCIO_BASE), "Jr" (port << 2));	\
	else								\
		__asm__ __volatile__(					\
		"str	%0, [%1, %2]"					\
		: : "r" (v|v<<16), "r" (IO_BASE), "r" (port << 2));	\
})
#endif

#define __outlc(value,port)						\
({									\
	if (__PORT_PCIO(port))						\
		__asm__ __volatile__(					\
		"str	%0, [%1, %2]"					\
		: : "r" (value), "r" (PCIO_BASE), "Jr" (port << 2));	\
	else								\
		__asm__ __volatile__(					\
		"str	%0, [%1, %2]"					\
		: : "r" (value), "r" (IO_BASE), "r" (port << 2));	\
})

#define __inbc(port)							\
({									\
	unsigned char result;						\
	if (__PORT_PCIO(port))						\
		__asm__ __volatile__(					\
		"ldrb	%0, [%1, %2]"					\
		: "=r" (result) : "r" (PCIO_BASE), "Jr" (port << 2));	\
	else								\
		__asm__ __volatile__(					\
		"ldrb	%0, [%1, %2]"					\
		: "=r" (result) : "r" (IO_BASE), "r" (port << 2));	\
	result;								\
})

#ifdef __HAVE_ASM_HALFWORD_LDST
#define __inwc(port)							\
({									\
	unsigned short result;						\
	if (port < 256)							\
		__asm__ __volatile__(					\
		"ldrh	%0, [%1, %2]"					\
		: "=r" (result) : "r" (PCIO_BASE), "J" (port << 2));	\
	else								\
	if (__PORT_PCIO(port))						\
		__asm__ __volatile__(					\
		"ldrh	%0, [%1, %2]"					\
		: "=r" (result) : "r" (PCIO_BASE), "r" (port << 2));	\
	else								\
		__asm__ __volatile__(					\
		"ldrh	%0, [%1, %2]"					\
		: "=r" (result) : "r" (IO_BASE), "r" (port << 2));	\
	result;								\
})
#else
#define __inwc(port)							\
({									\
	unsigned short result;						\
	if (__PORT_PCIO(port))						\
		__asm__ __volatile__(					\
		"ldr	%0, [%1, %2]"					\
		: "=r" (result) : "r" (PCIO_BASE), "Jr" (port << 2));	\
	else								\
		__asm__ __volatile__(					\
		"ldr	%0, [%1, %2]"					\
		: "=r" (result) : "r" (IO_BASE), "r" (port << 2));	\
	result & 0xffff;						\
})
#endif

#define __inlc(port)							\
({									\
	unsigned int result;						\
	if (__PORT_PCIO(port))						\
		__asm__ __volatile__(					\
		"ldr	%0, [%1, %2]"					\
		: "=r" (result) : "r" (PCIO_BASE), "Jr" (port << 2));	\
	else								\
		__asm__ __volatile__(					\
		"ldr	%0, [%1, %2]"					\
		: "=r" (result) : "r" (IO_BASE), "r" (port << 2));	\
	result;								\
})

#define __ioaddrc(port)							\
({									\
	unsigned long addr;						\
	if (__PORT_PCIO(port))						\
		addr = PCIO_BASE + (port << 2);				\
	else								\
		addr = IO_BASE + (port << 2);				\
	addr;								\
})

#define __OUTC(s,i,x)
#define __INC(s,i,x)

#else

/*
 * If we're not using the inline versions, we might as well use the assembler
 * versions.
 */
#define __outbc(v,p) __outb((v),(p))
#define __outwc(v,p) __outw((v),(p))
#define __outlc(v,p) __outl((v),(p))
#define __inbc(p) __inb((p))
#define __inwc(p) __inw((p))
#define __inlc(p) __inl((p))

#define __OUTC(s,i,x)
#define __INC(s,i,x)

#endif

#ifdef USE_INLINE_ASM1
extern __inline__ void __outb(unsigned int value, unsigned int port)
{
	unsigned long temp;
	__asm__ __volatile__(
	"tst	%2, #0x80000000
	moveq	%0, %3
	movne	%0, %4
	strb	%1, [%0, %2, lsl #2]"
	: "=&r" (temp) : "r" (value), "r" (port), "Ir" (PCIO_BASE), "Ir" (IO_BASE));
}

#ifdef __HAVE_HALFWORD_LDST
extern __inline__ void __outw(unsigned int value, unsigned int port)
{
	unsigned long temp;
	__asm__ __volatile__(
	"tst	%2, #0x80000000
	moveq	%0, %3
	movne	%0, %4
	strh	%1, [%0, %2, lsl #2]"
	: "=&r" (temp) : "r" (value), "r" (port), "Ir" (PCIO_BASE), "Ir" (IO_BASE));
}
#else
extern __inline__ void __outw(unsigned int value, unsigned int port)
{
	unsigned long temp;
	__asm__ __volatile__(
	"tst	%2, #0x80000000
	moveq	%0, %3
	movne	%0, %4
	str	%1, [%0, %2, lsl #2]"
	: "=&r" (temp) : "r" (value|value<<16), "r" (port), "Ir" (PCIO_BASE), "Ir" (IO_BASE));
}
#endif
	
extern __inline__ void __outl(unsigned int value, unsigned int port)
{
	unsigned long temp;
	__asm__ __volatile__(
	"tst	%2, #0x80000000
	moveq	%0, %3
	movne	%0, %4
	str	%1, [%0, %2, lsl #2]"
	: "=&r" (temp) : "r" (value), "r" (port), "Ir" (PCIO_BASE), "Ir" (IO_BASE));
}

extern __inline__ unsigned char __inb(unsigned int port)
{
	unsigned long temp, value;
	__asm__ __volatile__(
	"tst	%2, #0x80000000
	moveq	%0, %3
	movne	%0, %4
	ldrb	%1, [%0, %2, lsl #2]"
	: "=&r" (temp), "=r" (value) : "r" (port), "Ir" (PCIO_BASE), "Ir" (IO_BASE));
	return value;
}

#ifdef __HAVE_HALFWORD_LDST
extern __inline__ unsigned short __inw(unsigned int port)
{
	unsigned long temp, value;
	__asm__ __volatile__(
	"tst	%2, #0x80000000
	moveq	%0, %3
	movne	%0, %4
	ldrh	%1, [%0, %2, lsl #2]"
	: "=&r" (temp), "=r" (value) : "r" (port), "Ir" (PCIO_BASE), "Ir" (IO_BASE));
	return value;
}
#else
extern __inline__ unsigned short __inw(unsigned int port)
{
	unsigned long temp, value;
	__asm__ __volatile__(
	"tst	%2, #0x80000000
	moveq	%0, %3
	movne	%0, %4
	ldr	%1, [%0, %2, lsl #2]"
	: "=&r" (temp), "=r" (value) : "r" (port), "Ir" (PCIO_BASE), "Ir" (IO_BASE));
	return value;
}
#endif

extern __inline__ unsigned long __inl(unsigned int port)
{
	unsigned long temp, value;
	__asm__ __volatile__(
	"tst	%2, #0x80000000
	moveq	%0, %3
	movne	%0, %4
	ldr	%1, [%0, %2, lsl #2]"
	: "=&r" (temp), "=r" (value) : "r" (port), "Ir" (PCIO_BASE), "Ir" (IO_BASE));
	return value;
}

extern __inline__ unsigned long __ioaddr(unsigned int port)
{
	if (__PORT_PCIO(port))
		return (unsigned long)(PCIO_BASE + (port << 2));
	else
		return (unsigned long)(IO_BASE + (port << 2));
}

#define __OUT1(s,i,x)
#define __IN1(s,i,x)

#else

#define __OUT1(s,i,x) extern void __out##s(unsigned int value, unsigned int port);
#define __IN1(s,i,x)  extern unsigned x __in##s(unsigned int port);

#endif

#ifdef USE_INLINE_ASM
#define __INS(s,i,x) \
extern __inline__ void ins##s(unsigned int port, void *to, int len)			\
{											\
	unsigned x *p = (unsigned x *)to;						\
	if(__PORT_PCIO(port))								\
		port = PCIO_BASE+(port << 2);					\
	else										\
		port = IO_BASE+(port << 2);					\
	while(len-- > 0)								\
		*p++ = *(unsigned x *)port;						\
}

#define __OUTS(s,i,x) \
extern __inline__ void outs##s(unsigned int port, const void *from, int len)		\
{											\
	unsigned x *p = (unsigned x *)from;						\
	if(__PORT_PCIO(port))								\
		port = PCIO_BASE+(port << 2);					\
	else										\
		port = IO_BASE+(port << 2);					\
	while(len-- > 0)								\
		*(unsigned x *)port = *p++;						\
}

#else

#define __INS(s,i,x) \
extern void ins##s(unsigned int port, void *to, int len);

#define __OUTS(s,i,x) \
extern void outs##s(unsigned int port, const void *from, int len);

#endif

/*
 * These macros actually build the inline functions and/or the function prototypes
 */
#define __OUT(s,i,x) \
  __OUT1(s,i,x) \
  __OUTC(s,i,x) \
  __OUTS(s,i,x)

#define __IN(s,i,x) \
  __IN1(s,i,x) \
  __INC(s,i,x) \
  __INS(s,i,x)

#define __IO(s,i,x) \
  __OUT(s,i,x) \
  __IN(s,i,x)

__IO(b,"b",char)
__IO(w,"h",short)
__IO(l,"",long)

#define outb_p(val,port) outb((val),(port))
#define outw_p(val,port) outw((val),(port))
#define outl_p(val,port) outl((val),(port))
#define inb_p(port) inb((port))
#define inw_p(port) inw((port))
#define inl_p(port) inl((port))

#define outsb_p(port,from,len) outsb(port,from,len)
#define outsw_p(port,from,len) outsw(port,from,len)
#define outsl_p(port,from,len) outsl(port,from,len)
#define insb_p(port,to,len) insb(port,to,len)
#define insw_p(port,to,len) insw(port,to,len)
#define insl_p(port,to,len) insl(port,to,len)

#define outb(val,port)	\
  (__builtin_constant_p((port)) ? __outbc((val),(port)) : __outb((val),(port)))

#define outw(val,port)	\
  (__builtin_constant_p((port)) ? __outwc((val),(port)) : __outw((val),(port)))

#define outl(val,port)	\
  (__builtin_constant_p((port)) ? __outlc((val),(port)) : __outl((val),(port)))

#define inb(port)	\
  (__builtin_constant_p((port)) ? __inbc((port)) : __inb((port)))

#define inw(port)	\
  (__builtin_constant_p((port)) ? __inwc((port)) : __inw((port)))

#define inl(port)	\
  (__builtin_constant_p((port)) ? __inlc((port)) : __inl((port)))

#define ioaddr(port)	\
  (__builtin_constant_p((port)) ? __ioaddrc((port)) : __ioaddr((port)))
  
#endif

